"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const dgram_1 = require("dgram");
const dotup_ts_types_1 = require("dotup-ts-types");
const events_1 = require("events");
const os_1 = __importDefault(require("os"));
const SsdpMessage_1 = require("../Messages/SsdpMessage");
const SsdpConstants_1 = require("../SsdpConstants");
const tools_1 = require("../tools");
const SsdpSocketEvents_1 = require("./SsdpSocketEvents");
const logger = tools_1.getNamedLogger(tools_1.LOGGER_NAME);
class SsdpSocket extends events_1.EventEmitter {
    constructor(...interfaceNames) {
        super();
        this.interfaceNames = interfaceNames.length > 0 ? interfaceNames : undefined;
    }
    publish(message) {
        this.sockets.forEach(s => {
            const data = dotup_ts_types_1.StringTools.format(message, { ip: s.ip, hostname: tools_1.getHostname() });
            s.send(data, SsdpConstants_1.SsdpConstants.port, SsdpConstants_1.SsdpConstants.ipv4Address, (e, b) => {
                if (e !== undefined && e !== null) {
                    logger.error(e);
                }
            });
        });
    }
    isConnected() {
        if (this.sockets === undefined || this.sockets.length < 0) {
            return false;
        }
        return this.sockets.every(s => s.connected);
    }
    initialize() {
        const interfaces = os_1.default.networkInterfaces();
        const allKeys = Object.keys(interfaces);
        let keys = allKeys;
        if (this.sockets === undefined) {
            this.sockets = [];
        }
        else {
            logger.warn('Already initialized');
            return;
        }
        if (this.interfaceNames !== undefined) {
            const notFound = this.interfaceNames.filter(name => interfaces[name] === undefined);
            notFound.forEach(name => logger.warn(`Interface '${name}' not found.`));
            keys = allKeys.filter(name => this.interfaceNames.includes(name));
        }
        if (!this.hasIp4(keys)) {
            logger.fatal('No ipv4 interface available');
            process.exit(8001);
        }
        keys.forEach(name => {
            this.createSocket(name, interfaces[name]);
        });
        setTimeout(() => {
            if (!this.isConnected()) {
                logger.warn('Socket bind failed.');
                this.dispose();
                this.initialize();
            }
        }, 5000);
    }
    hasIp4(keys) {
        const interfaces = os_1.default.networkInterfaces();
        return keys.some(key => {
            const netInfo = interfaces[key];
            return netInfo.some(net => net.family === 'IPv4');
        });
    }
    createSocket(name, netInfo) {
        netInfo.forEach(info => {
            logger.info(`netInfo:${JSON.stringify(info)}`);
            if (info.internal || info.family === 'IPv6') {
                return;
            }
            const opt = {
                type: 'udp4',
                reuseAddr: true
            };
            logger.info(`Try create socket ${name}(${info.address})`);
            const socket = dgram_1.createSocket(opt);
            socket.ip = info.address;
            socket.interfaceName = name;
            socket.connected = false;
            logger.info(`Socket ${name}(${socket.ip}) created`);
            socket.on('message', (m, r) => {
                this.emit(SsdpSocketEvents_1.SsdpSocketEvents.message, m, r);
                this.onMessage(m, r);
            });
            socket.on('error', e => {
                logger.error(e);
                this.emit(SsdpSocketEvents_1.SsdpSocketEvents.error, e);
            });
            socket.on('listening', () => {
                socket.connected = true;
                this.emit(SsdpSocketEvents_1.SsdpSocketEvents.listening, socket);
                logger.info(`Socket ${socket.ip} listening`);
            });
            socket.bind(SsdpConstants_1.SsdpConstants.port, () => {
                socket.addMembership(SsdpConstants_1.SsdpConstants.ipv4Address, info.address);
                socket.setMulticastTTL(SsdpConstants_1.SsdpConstants.ttl);
                socket.setMulticastLoopback(true);
                socket.setBroadcast(true);
                logger.info(`Socket ${socket.ip} configured`);
            });
            this.sockets.push(socket);
        });
    }
    onMessage(msg, rinfo) {
        logger.debug(msg.toString());
        const rows = msg
            .toString()
            .split('\r\n');
        const m = new SsdpMessage_1.SsdpMessage();
        // tslint:disable-next-line: no-any
        const startLine = rows.shift();
        rows.forEach(row => {
            if (row.trim().length > 0) {
                const split = row.indexOf(':');
                const key = row
                    .substr(0, split)
                    .trim()
                    .toUpperCase();
                const value = row
                    .substr(split + 1)
                    .trim();
                // tslint:disable-next-line: no-any
                m[key] = value;
            }
        });
        switch (startLine) {
            case SsdpConstants_1.SsdpConstants.StartLine['M-SEARCH']:
                this.emit(SsdpSocketEvents_1.SsdpSocketEvents.msearch, m, rinfo);
                break;
            case SsdpConstants_1.SsdpConstants.StartLine.NOTIFY:
                this.emit(SsdpSocketEvents_1.SsdpSocketEvents.notify, m);
        }
    }
    // tslint:disable-next-line: no-any
    on(event, listener) {
        return super.on(event, listener);
    }
    dispose() {
        if (this.sockets !== undefined) {
            this.sockets.forEach(s => {
                s.removeAllListeners();
                s.close();
                s.unref();
            });
            this.sockets = undefined;
        }
    }
}
exports.SsdpSocket = SsdpSocket;

//# sourceMappingURL=SsdpSocket.js.map
